@@ -40,6 +40,7 @@
 #include "photos-camera-cache.h"
 #include "photos-dlna-renderers-dialog.h"
 #include "photos-export-dialog.h"
+#include "photos-export-notification.h"
 #include "photos-filterable.h"
 #include "photos-gom-miner.h"
 #include "photos-item-manager.h"
@@ -95,7 +96,6 @@ struct _PhotosApplicationPrivate
   GSimpleAction *set_bg_action;
   GSimpleAction *set_ss_action;
   GSimpleAction *sharpen_action;
-  GSimpleAction *undo_action;
   GtkWidget *main_window;
   PhotosCameraCache *camera_cache;
   PhotosSearchContextState *state;
@@ -319,7 +319,6 @@ photos_application_actions_update (PhotosApplication *self)
   g_simple_action_set_enabled (priv->insta_action, enable);
   g_simple_action_set_enabled (priv->saturation_action, enable);
   g_simple_action_set_enabled (priv->sharpen_action, enable);
-  g_simple_action_set_enabled (priv->undo_action, enable);
 
   enable = (mode == PHOTOS_WINDOW_MODE_COLLECTIONS
             || mode == PHOTOS_WINDOW_MODE_FAVORITES
@@ -341,10 +340,13 @@ photos_application_actions_update (PhotosApplication *self)
 
   enable = (load_state == PHOTOS_LOAD_STATE_FINISHED && mode == PHOTOS_WINDOW_MODE_PREVIEW);
   g_simple_action_set_enabled (priv->gear_action, enable);
-  g_simple_action_set_enabled (priv->properties_action, enable);
   g_simple_action_set_enabled (priv->set_bg_action, enable);
   g_simple_action_set_enabled (priv->set_ss_action, enable);
 
+  enable = ((load_state == PHOTOS_LOAD_STATE_FINISHED && mode == PHOTOS_WINDOW_MODE_PREVIEW)
+            || (selection_mode && item != NULL));
+  g_simple_action_set_enabled (priv->properties_action, enable);
+
   enable = ((load_state == PHOTOS_LOAD_STATE_FINISHED && mode == PHOTOS_WINDOW_MODE_PREVIEW)
             || (selection_mode && item != NULL && !photos_base_item_is_collection (item)));
   g_simple_action_set_enabled (priv->print_action, enable);
@@ -724,8 +726,7 @@ photos_application_edit_cancel (PhotosApplication *self)
   item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
   g_return_if_fail (item != NULL);
 
-  while (photos_base_item_operation_undo (item))
-    ;
+  photos_base_item_operations_revert (item);
 
   g_application_hold (G_APPLICATION (self));
   photos_base_item_process_async (item, NULL, photos_application_edit_cancel_process, self);
@@ -741,6 +742,7 @@ photos_application_edit_current (PhotosApplication *self)
   item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
   g_return_if_fail (item != NULL);
 
+  photos_base_item_pipeline_snapshot (item);
   photos_mode_controller_set_window_mode (priv->state->mode_cntrlr, PHOTOS_WINDOW_MODE_EDIT);
 }
 
@@ -843,21 +845,31 @@ photos_application_print_current (PhotosApplication *self)
 }
 
 
+static void
+photos_application_properties_response (GtkDialog *dialog, gint response_id, gpointer user_data)
+{
+  PhotosApplication *self = PHOTOS_APPLICATION (user_data);
+
+  gtk_widget_destroy (GTK_WIDGET (dialog));
+  photos_selection_controller_set_selection_mode (self->priv->sel_cntrlr, FALSE);
+}
+
+
 static void
 photos_application_properties (PhotosApplication *self)
 {
   PhotosApplicationPrivate *priv = self->priv;
-  GObject *item;
   GtkWidget *dialog;
+  PhotosBaseItem *item;
   const gchar *id;
 
-  item = photos_base_manager_get_active_object (priv->state->item_mngr);
+  item = photos_application_get_selection_or_active_item (self);
   g_return_if_fail (item != NULL);
 
   id = photos_filterable_get_id (PHOTOS_FILTERABLE (item));
   dialog = photos_properties_dialog_new (GTK_WINDOW (priv->main_window), id);
   gtk_widget_show_all (dialog);
-  g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
+  g_signal_connect (dialog, "response", G_CALLBACK (photos_application_properties_response), self);
 }
 
 
@@ -981,18 +993,26 @@ photos_application_save_save (GObject *source_object, GAsyncResult *res, gpointe
   PhotosApplication *self = PHOTOS_APPLICATION (user_data);
   PhotosBaseItem *item = PHOTOS_BASE_ITEM (source_object);
   GError *error = NULL;
+  GFile *file = NULL;
+  GList *items = NULL;
 
-  photos_base_item_save_finish (item, res, &error);
+  file = photos_base_item_save_finish (item, res, &error);
   if (error != NULL)
     {
       g_warning ("Unable to save: %s", error->message);
+      photos_export_notification_new_with_error (error);
       g_error_free (error);
       goto out;
     }
 
+  items = g_list_prepend (items, g_object_ref (item));
+  photos_export_notification_new (items, file);
+
  out:
   photos_selection_controller_set_selection_mode (self->priv->sel_cntrlr, FALSE);
   g_application_release (G_APPLICATION (self));
+  g_clear_object (&file);
+  g_list_free_full (items, g_object_unref);
 }
 
 
@@ -1023,6 +1043,7 @@ photos_application_save_response (GtkDialog *dialog, gint response_id, gpointer
   if (!photos_utils_make_directory_with_parents (export, NULL, &error))
     {
       g_warning ("Unable to create %s: %s", export_path, error->message);
+      photos_export_notification_new_with_error (error);
       g_error_free (error);
       goto out;
     }
@@ -1034,6 +1055,7 @@ photos_application_save_response (GtkDialog *dialog, gint response_id, gpointer
   if (error != NULL)
     {
       g_warning ("Unable to get a child for %s: %s", export_dir_name, error->message);
+      photos_export_notification_new_with_error (error);
       g_error_free (error);
       goto out;
     }
@@ -1045,6 +1067,7 @@ photos_application_save_response (GtkDialog *dialog, gint response_id, gpointer
   if (!photos_utils_make_directory_with_parents (export, NULL, &error))
     {
       g_warning ("Unable to create %s: %s", export_path, error->message);
+      photos_export_notification_new_with_error (error);
       g_error_free (error);
       goto out;
     }
@@ -1536,9 +1559,6 @@ photos_application_startup (GApplication *application)
   priv->sharpen_action = g_simple_action_new ("sharpen-current", G_VARIANT_TYPE_DOUBLE);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->sharpen_action));
 
-  priv->undo_action = g_simple_action_new ("undo-current", NULL);
-  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->undo_action));
-
   g_signal_connect_swapped (priv->state->mode_cntrlr,
                             "window-mode-changed",
                             G_CALLBACK (photos_application_window_mode_changed),
@@ -1641,7 +1661,6 @@ photos_application_dispose (GObject *object)
   g_clear_object (&priv->set_bg_action);
   g_clear_object (&priv->set_ss_action);
   g_clear_object (&priv->sharpen_action);
-  g_clear_object (&priv->undo_action);
   g_clear_object (&priv->camera_cache);
   g_clear_object (&priv->sel_cntrlr);
   g_clear_object (&priv->extract_priority);
