@@ -488,7 +488,7 @@ photos_base_item_download_in_thread_func (GTask *task,
   PhotosBaseItem *self = PHOTOS_BASE_ITEM (source_object);
   PhotosBaseItemPrivate *priv = self->priv;
   GError *error;
-  gchar *path;
+  gchar *path = NULL;
 
   g_mutex_lock (&priv->mutex_download);
 
@@ -500,9 +500,10 @@ photos_base_item_download_in_thread_func (GTask *task,
       goto out;
     }
 
-  g_task_return_pointer (task, path, g_free);
+  g_task_return_pointer (task, g_strdup (path), g_free);
 
  out:
+  g_free (path);
   g_mutex_unlock (&priv->mutex_download);
 }
 
@@ -794,7 +795,7 @@ photos_base_item_load_buffer_in_thread_func (GTask *task,
 {
   PhotosBaseItem *self = PHOTOS_BASE_ITEM (source_object);
   PhotosBaseItemPrivate *priv = self->priv;
-  GeglBuffer *buffer;
+  GeglBuffer *buffer = NULL;
   GError *error = NULL;
 
   g_mutex_lock (&priv->mutex);
@@ -806,9 +807,10 @@ photos_base_item_load_buffer_in_thread_func (GTask *task,
       goto out;
     }
 
-  g_task_return_pointer (task, buffer, g_object_unref);
+  g_task_return_pointer (task, g_object_ref (buffer), g_object_unref);
 
  out:
+  g_clear_object (&buffer);
   g_mutex_unlock (&priv->mutex);
 }
 
@@ -1145,7 +1147,11 @@ photos_base_item_save_save_metadata (GObject *source_object, GAsyncResult *res,
 {
   PhotosBaseItem *self = PHOTOS_BASE_ITEM (source_object);
   GError *error;
+  GFile *file = NULL;
   GTask *task = G_TASK (user_data);
+  PhotosBaseItemSaveData *data;
+
+  data = (PhotosBaseItemSaveData *) g_task_get_task_data (task);
 
   error = NULL;
   if (!photos_base_item_save_metadata_finish (self, res, &error))
@@ -1154,9 +1160,11 @@ photos_base_item_save_save_metadata (GObject *source_object, GAsyncResult *res,
       goto out;
     }
 
-  g_task_return_boolean (task, TRUE);
+  file = g_file_get_child (data->dir, self->priv->filename);
+  g_task_return_pointer (task, g_object_ref (file), g_object_unref);
 
  out:
+  g_clear_object (&file);
   g_object_unref (task);
 }
 
@@ -2170,9 +2178,16 @@ photos_base_item_operation_get (PhotosBaseItem *self, const gchar *operation, co
 
 
 gboolean
-photos_base_item_operation_undo (PhotosBaseItem *self)
+photos_base_item_operation_remove (PhotosBaseItem *self, const gchar *operation)
 {
-  return photos_pipeline_undo (self->priv->pipeline);
+  return photos_pipeline_remove (self->priv->pipeline, operation);
+}
+
+
+void
+photos_base_item_operations_revert (PhotosBaseItem *self)
+{
+  return photos_pipeline_revert (self->priv->pipeline);
 }
 
 
@@ -2217,6 +2232,13 @@ photos_base_item_pipeline_save_finish (PhotosBaseItem *self, GAsyncResult *res,
 }
 
 
+void
+photos_base_item_pipeline_snapshot (PhotosBaseItem *self)
+{
+  return photos_pipeline_snapshot (self->priv->pipeline);
+}
+
+
 void
 photos_base_item_print (PhotosBaseItem *self, GtkWidget *toplevel)
 {
@@ -2331,7 +2353,7 @@ photos_base_item_save_async (PhotosBaseItem *self,
 }
 
 
-gboolean
+GFile *
 photos_base_item_save_finish (PhotosBaseItem *self, GAsyncResult *res, GError **error)
 {
   GTask *task = G_TASK (res);
@@ -2340,7 +2362,7 @@ photos_base_item_save_finish (PhotosBaseItem *self, GAsyncResult *res, GError **
   g_return_val_if_fail (g_task_get_source_tag (task) == photos_base_item_save_async, FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  return g_task_propagate_boolean (task, error);
+  return g_task_propagate_pointer (task, error);
 }
 
 
