@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2015, 2016 Red Hat, Inc.
+ * Copyright © 2015 – 2016 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -104,9 +104,9 @@ photos_operation_insta_filter_setup (PhotosOperationInstaFilter *self)
       self->nodes = g_list_prepend (self->nodes, node);
       break;
 
-    case PHOTOS_OPERATION_INSTA_PRESET_GRAY:
+    case PHOTOS_OPERATION_INSTA_PRESET_HEFE:
       node = gegl_node_new_child (operation->node,
-                                  "operation", "gegl:gray",
+                                  "operation", "photos:insta-hefe",
                                   NULL);
       self->nodes = g_list_prepend (self->nodes, node);
       break;
