@@ -1,6 +1,7 @@
 /*
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2015 – 2016 Red Hat, Inc.
+ * Copyright © 2016 Umang Jain
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -36,8 +37,8 @@ struct _PhotosPipeline
   GObject parent_instance;
   GeglNode *parent;
   GHashTable *hash;
-  GQueue *history;
   GeglNode *graph;
+  gchar *snapshot;
   gchar *uri;
 };
 
@@ -82,6 +83,79 @@ photos_pipeline_reset (PhotosPipeline *self)
 }
 
 
+static gboolean
+photos_pipeline_create_graph_from_xml (PhotosPipeline *self, gchar *contents)
+{
+  GeglNode *graph = NULL;
+  GeglNode *input;
+  GeglNode *node;
+  GeglNode *output;
+  GSList *children = NULL;
+  GSList *l;
+  gboolean ret_val = FALSE;
+
+  /* HACK: This graph is busted. eg., the input and output proxies
+   * point to the same GeglNode. I can't imagine this to be
+   * anything else other than a GEGL bug.
+   *
+   * Therefore, we are going to re-construct a proper graph
+   * ourselves.
+   */
+  graph = gegl_node_new_from_xml (contents, "/");
+  if (graph == NULL)
+    goto out;
+
+  g_hash_table_remove_all (self->hash);
+  photos_utils_remove_children_from_node (self->graph);
+
+  input = gegl_node_get_input_proxy (self->graph, "input");
+  output = gegl_node_get_output_proxy (self->graph, "output");
+
+  children = gegl_node_get_children (graph);
+  if (children == NULL)
+    {
+      gegl_node_link (input, output);
+      goto carry_on;
+    }
+
+  for (l = children; l != NULL; l = l->next)
+    {
+      const gchar *operation;
+
+      node = GEGL_NODE (l->data);
+
+      g_object_ref (node);
+      gegl_node_remove_child (graph, node);
+      gegl_node_add_child (self->graph, node);
+      g_object_unref (node);
+
+      operation = gegl_node_get_operation (node);
+      g_hash_table_insert (self->hash, g_strdup (operation), g_object_ref (node));
+    }
+
+  node = GEGL_NODE (children->data);
+  gegl_node_link (input, node);
+
+  for (l = children; l != NULL && l->next != NULL; l = l->next)
+    {
+      GeglNode *sink = GEGL_NODE (l->next->data);
+      GeglNode *source = GEGL_NODE (l->data);
+      gegl_node_link (source, sink);
+    }
+
+  node = GEGL_NODE (l->data);
+  gegl_node_link (node, output);
+
+ carry_on:
+  ret_val = TRUE;
+
+ out:
+  g_slist_free (children);
+  g_clear_object (&graph);
+  return ret_val;
+}
+
+
 static void
 photos_pipeline_save_replace_contents (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
@@ -148,7 +222,7 @@ photos_pipeline_finalize (GObject *object)
 {
   PhotosPipeline *self = PHOTOS_PIPELINE (object);
 
-  g_queue_free (self->history);
+  g_free (self->snapshot);
   g_free (self->uri);
 
   G_OBJECT_CLASS (photos_pipeline_parent_class)->finalize (object);
@@ -181,7 +255,6 @@ static void
 photos_pipeline_init (PhotosPipeline *self)
 {
   self->hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
-  self->history = g_queue_new ();
 }
 
 
@@ -218,15 +291,8 @@ photos_pipeline_async_initable_init_load_contents (GObject *source_object, GAsyn
 {
   GTask *task = G_TASK (user_data);
   PhotosPipeline *self;
-  GeglNode *graph = NULL;
-  GeglNode *input;
-  GeglNode *node;
-  GeglNode *output;
-  GeglNode *parent;
   GError *error;
   GFile *file = G_FILE (source_object);
-  GSList *children = NULL;
-  GSList *l;
   gchar *contents = NULL;
 
   self = PHOTOS_PIPELINE (g_task_get_source_object (task));
@@ -246,76 +312,20 @@ photos_pipeline_async_initable_init_load_contents (GObject *source_object, GAsyn
         }
     }
 
-  /* HACK: This graph is busted. eg., the input and output proxies
-   * point to the same GeglNode. I can't imagine this to be
-   * anything else other than a GEGL bug.
-   *
-   * Therefore, we are going to re-construct a proper graph
-   * ourselves.
-   */
-  graph = gegl_node_new_from_xml (contents, "/");
-  if (graph == NULL)
+  if (!(photos_pipeline_create_graph_from_xml (self, contents)))
     {
       gchar *uri;
 
       uri = g_file_get_uri (file);
       g_warning ("Unable to deserialize from %s", uri);
       g_free (uri);
-      goto carry_on;
     }
 
-  parent = gegl_node_get_parent (self->graph);
-  g_clear_object (&self->graph);
-
-  self->graph = gegl_node_new ();
-  if (parent != NULL)
-    gegl_node_add_child (parent, self->graph);
-
-  input = gegl_node_get_input_proxy (self->graph, "input");
-  output = gegl_node_get_output_proxy (self->graph, "output");
-
-  children = gegl_node_get_children (graph);
-  if (children == NULL)
-    {
-      gegl_node_link (input, output);
-      goto carry_on;
-    }
-
-  for (l = children; l != NULL; l = l->next)
-    {
-      const gchar *operation;
-
-      node = GEGL_NODE (l->data);
-
-      g_object_ref (node);
-      gegl_node_remove_child (graph, node);
-      gegl_node_add_child (self->graph, node);
-      g_object_unref (node);
-
-      operation = gegl_node_get_operation (node);
-      g_hash_table_insert (self->hash, g_strdup (operation), g_object_ref (node));
-    }
-
-  node = GEGL_NODE (children->data);
-  gegl_node_link (input, node);
-
-  for (l = children; l != NULL && l->next != NULL; l = l->next)
-    {
-      GeglNode *sink = GEGL_NODE (l->next->data);
-      GeglNode *source = GEGL_NODE (l->data);
-      gegl_node_link (source, sink);
-    }
-
-  node = GEGL_NODE (l->data);
-  gegl_node_link (node, output);
-
  carry_on:
   g_task_return_boolean (task, TRUE);
 
  out:
   g_free (contents);
-  g_slist_free (children);
-  g_clear_object (&graph);
   g_object_unref (task);
 }
 
@@ -435,10 +445,6 @@ photos_pipeline_add (PhotosPipeline *self, const gchar *operation, const gchar *
   xml = gegl_node_to_xml_full (self->graph, self->graph, "/");
   photos_debug (PHOTOS_DEBUG_GEGL, "Pipeline: %s", xml);
 
-  /* We want to remove the nodes from the graph too. */
-  g_queue_free_full (self->history, g_object_unref);
-  self->history = g_queue_new ();
-
   g_free (xml);
 }
 
@@ -541,30 +547,28 @@ photos_pipeline_save_finish (PhotosPipeline *self, GAsyncResult *res, GError **e
 
 
 gboolean
-photos_pipeline_undo (PhotosPipeline *self)
+photos_pipeline_remove (PhotosPipeline *self, const gchar *operation)
 {
-  GeglNode *input;
-  GeglNode *last;
-  GeglNode *last2;
-  GeglNode *output;
+  GeglNode *node;
+  GeglNode *source;
+  GeglNode **sinks = NULL;
   gboolean ret_val = FALSE;
-  gchar *operation = NULL;
   gchar *xml = NULL;
+  gint i;
+  gint nsinks;
 
-  input = gegl_node_get_input_proxy (self->graph, "input");
-  output = gegl_node_get_output_proxy (self->graph, "output");
-  last = gegl_node_get_producer (output, "input", NULL);
-  if (last == input)
+  node = GEGL_NODE (g_hash_table_lookup (self->hash, operation));
+  if (node == NULL)
     goto out;
 
-  gegl_node_get (last, "operation", &operation, NULL);
-  g_hash_table_remove (self->hash, operation);
-  g_queue_push_head (self->history, last);
+  source = gegl_node_get_producer (node, "input", NULL);
+  nsinks = gegl_node_get_consumers (node, "output", &sinks, NULL);
 
-  last2 = gegl_node_get_producer (last, "input", NULL);
-  gegl_node_disconnect (output, "input");
-  gegl_node_disconnect (last, "input");
-  gegl_node_link (last2, output);
+  g_hash_table_remove (self->hash, operation);
+  gegl_node_remove_child (self->graph, node);
+
+  for (i = 0; i < nsinks; i++)
+    gegl_node_link (source, sinks[i]);
 
   xml = gegl_node_to_xml_full (self->graph, self->graph, "/");
   photos_debug (PHOTOS_DEBUG_GEGL, "Pipeline: %s", xml);
@@ -572,7 +576,35 @@ photos_pipeline_undo (PhotosPipeline *self)
   ret_val = TRUE;
 
  out:
+  g_free (sinks);
   g_free (xml);
-  g_free (operation);
   return ret_val;
 }
+
+
+void
+photos_pipeline_revert (PhotosPipeline *self)
+{
+  gchar *xml;
+
+  g_return_if_fail (self->snapshot != NULL);
+
+  if (!photos_pipeline_create_graph_from_xml (self, self->snapshot))
+    g_warning ("Unable to revert to: %s", self->snapshot);
+
+  g_clear_pointer (&self->snapshot, g_free);
+
+  xml = gegl_node_to_xml_full (self->graph, self->graph, "/");
+  photos_debug (PHOTOS_DEBUG_GEGL, "Pipeline: %s", xml);
+
+  g_free (xml);
+}
+
+
+void
+photos_pipeline_snapshot (PhotosPipeline *self)
+{
+  g_free (self->snapshot);
+  self->snapshot = gegl_node_to_xml_full (self->graph, self->graph, "/");
+  photos_debug (PHOTOS_DEBUG_GEGL, "Snapshot: %s", self->snapshot);
+}
