@@ -546,25 +546,20 @@ photos_utils_create_symbolic_icon_for_scale (const gchar *name, gint base_size,
 
 
 gboolean
-photos_utils_create_thumbnail (GFile *file, GCancellable *cancellable, GError **error)
+photos_utils_create_thumbnail (GFile *file,
+                               const gchar *mime_type,
+                               gint64 mtime,
+                               GCancellable *cancellable,
+                               GError **error)
 {
   GnomeDesktopThumbnailFactory *factory = NULL;
-  GFileInfo *info = NULL;
-  const gchar *attributes = G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE","G_FILE_ATTRIBUTE_TIME_MODIFIED;
   gboolean ret_val = FALSE;
   gchar *uri = NULL;
   GdkPixbuf *pixbuf = NULL;
-  guint64 mtime;
 
   uri = g_file_get_uri (file);
-  info = g_file_query_info (file, attributes, G_FILE_QUERY_INFO_NONE, cancellable, error);
-  if (info == NULL)
-    goto out;
-
-  mtime = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
-
   factory = gnome_desktop_thumbnail_factory_new (GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE);
-  pixbuf = gnome_desktop_thumbnail_factory_generate_thumbnail (factory, uri, g_file_info_get_content_type (info));
+  pixbuf = gnome_desktop_thumbnail_factory_generate_thumbnail (factory, uri, mime_type);
   if (pixbuf == NULL)
     {
       /* FIXME: use proper #defines and enumerated types */
@@ -581,7 +576,6 @@ photos_utils_create_thumbnail (GFile *file, GCancellable *cancellable, GError **
  out:
   g_clear_object (&pixbuf);
   g_clear_object (&factory);
-  g_clear_object (&info);
   g_free (uri);
   return ret_val;
 }
@@ -1200,6 +1194,34 @@ photos_utils_update_executed (GObject *source_object, GAsyncResult *res, gpointe
 }
 
 
+void
+photos_utils_remove_children_from_node (GeglNode *node)
+{
+  GeglNode *input;
+  GeglNode *last;
+  GeglNode *output;
+  GeglOperation *operation;
+
+  operation = gegl_node_get_gegl_operation (node);
+  g_return_if_fail (operation == NULL);
+
+  input = gegl_node_get_input_proxy (node, "input");
+  output = gegl_node_get_output_proxy (node, "output");
+  last = gegl_node_get_producer (output, "input", NULL);
+
+  while (last != NULL && last != input)
+    {
+      GeglNode *last2;
+
+      last2 = gegl_node_get_producer (last, "input", NULL);
+      gegl_node_remove_child (node, last);
+      last = last2;
+    }
+
+  gegl_node_link (input, output);
+}
+
+
 void
 photos_utils_set_edited_name (const gchar *urn, const gchar *title)
 {
