@@ -83,7 +83,11 @@ GdkPixbuf       *photos_utils_create_placeholder_icon_for_scale (const gchar *na
 
 GIcon           *photos_utils_create_symbolic_icon_for_scale (const gchar *name, gint base_size, gint scale);
 
-gboolean         photos_utils_create_thumbnail            (GFile *file, GCancellable *cancellable, GError **error);
+gboolean         photos_utils_create_thumbnail            (GFile *file,
+                                                           const gchar *mime_type,
+                                                           gint64 mtime,
+                                                           GCancellable *cancellable,
+                                                           GError **error);
 
 GIcon           *photos_utils_get_icon_from_cursor        (TrackerSparqlCursor *cursor);
 
@@ -153,6 +157,8 @@ GQuark           photos_utils_orientation_right_quark     (void);
 
 GQuark           photos_utils_orientation_top_quark       (void);
 
+void             photos_utils_remove_children_from_node   (GeglNode *node);
+
 void             photos_utils_set_edited_name             (const gchar *urn, const gchar *title);
 
 void             photos_utils_set_favorite                (const gchar *urn, gboolean is_favorite);
